@@ -2,7 +2,7 @@ package HealthCheck::Diagnostic::DBHCheck;
 
 # ABSTRACT: Check a database handle to make sure you have read/write access
 use version;
-our $VERSION = 'v0.500.3'; # VERSION
+our $VERSION = 'v1.0.0'; # VERSION
 
 use 5.010;
 use strict;
@@ -49,7 +49,30 @@ sub check {
     croak "The value '$db_access' is not valid for the 'db_access' parameter"
         unless $db_access =~ /^r[ow]$/;
 
-    $dbh = $dbh->(%params);
+    my $timeout =
+        defined $params{timeout}                  ? $params{timeout} :
+        (ref $self) && (defined $self->{timeout}) ? $self->{timeout} :
+                                                    10;
+
+    local $@;
+    eval {
+        local $SIG{__DIE__};
+        local $SIG{ALRM} = sub { die "timeout after $timeout seconds.\n" };
+        alarm $timeout;
+        $dbh = $dbh->(%params);
+    };
+    alarm 0;
+
+    if ( $@ =~ /^timeout/ ) {
+        chomp $@;
+        return {
+            status => 'CRITICAL',
+            info   => "Database connection $@",
+        };
+    }
+
+    # re-throw any other exceptions
+    die $@ if $@;
 
     croak "The 'dbh' coderef should return an object!"
         unless blessed $dbh;
@@ -169,7 +192,7 @@ HealthCheck::Diagnostic::DBHCheck - Check a database handle to make sure you hav
 
 =head1 VERSION
 
-version v0.500.3
+version v1.0.0
 
 =head1 SYNOPSIS
 
@@ -178,6 +201,7 @@ version v0.500.3
             dbh       => \&connect_to_read_write_db,
             db_access => "rw",
             tags      => [qw< dbh_check_rw >]
+            timeout   => 10, # default
         ),
         HealthCheck::Diagnostic::DBHCheck->new(
             dbh       => \&connect_to_read_only_db,
@@ -204,12 +228,12 @@ Those inherited from L<HealthCheck::Diagnostic/ATTRIBUTES> plus:
 
 =head2 label
 
-Inherited from L<HealthCheck::Diagnostic/label1>,
+Inherited from L<"label" in HealthCheck::Diagnostic|HealthCheck::Diagnostic/label1>,
 defaults to C<dbh_check>.
 
 =head2 tags
 
-Inherited from L<HealthCheck::Diagnostic/tags1>,
+Inherited from L<"tags" in HealthCheck::Diagnostic|HealthCheck::Diagnostic/tags1>,
 defaults to C<[ 'dbh_check' ]>.
 
 =head2 dbh
@@ -236,6 +260,13 @@ The expected class for the database handle returned by the C<dbh> coderef.
 
 Defaults to C<DBI::db>.
 
+=head2 timeout
+
+Sets up an C<ALRM> signal handler used to timeout the initial connection
+attempt after the number of seconds provided.
+
+Defaults to 10.
+
 =head1 DEPENDENCIES
 
 L<HealthCheck::Diagnostic>
@@ -250,7 +281,7 @@ Grant Street Group <developers@grantstreet.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2019 - 2020 by Grant Street Group.
+This software is Copyright (c) 2019 - 2023 by Grant Street Group.
 
 This is free software, licensed under:
 
